<?xml version="1.0" encoding="ASCII"?>
<!--
The contents of this file are subject to the Health Level-7 Public
License Version 1.0 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of the
License at http://www.hl7.org/HPL/hpl.txt.

Software distributed under the License is distributed on an "AS IS"
basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
the License for the specific language governing rights and
limitations under the License.

The Original Code is all this file.

The Initial Developer of the Original Code is Gunther Schadow.
Portions created by Initial Developer are Copyright (C) 2002-2004
Health Level Seven, Inc. All Rights Reserved.

Contributor(s): Steven Gitterman, Brian Keller

Revision: $Id: spl2.xsl,v 1.14 2005/03/09 00:47:39 gschadow Exp $
-->
<xsl:transform version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:v3="urn:hl7-org:v3"
  xmlns:exsl="http://exslt.org/common"
  xmlns:msxsl="urn:schemas-microsoft-com:xslt"
  exclude-result-prefixes="exsl msxsl v3 xsl">

  <xsl:import href="xml-verbatim.xsl"/>
  <xsl:param name="show-subjects-xml" select="/.."/>
  <xsl:param name="show-data" select="/.."/>
  <xsl:param name="standardSections" select="document('spl2-sections.xml')/*"/>
  <xsl:param name="footnoteMarks" select="'*&#8224;&#8225;&#167;&#182;#&#9824;&#9829;&#9830;&#9827;'"/>

  <xsl:output method="html" version="1.0" encoding="UTF-8" indent="yes"/>
  <xsl:strip-space elements="*"/>

  <!-- TOPLEVEL -->

  <xsl:template match="/">
    <html>
      <head>
        <title><xsl:value-of select="v3:document/v3:title"/></title>
        <link rel="stylesheet" type="text/css" href="spl2.css"/>
	<xsl:if test="$show-subjects-xml">
	  <xsl:call-template name="xml-verbatim-setup"/>
	</xsl:if>	  
      </head>
      <body>
        <xsl:apply-templates select="@*|node()"/>
	<xsl:if test="$show-subjects-xml">
	  <hr/>
          <div class="Subject"
               onclick="xmlVerbatimClick(event);"
               ondblclick="xmlVerbatimDblClick(event);">
             <xsl:apply-templates mode="xml-verbatim" select="node()"/>
          </div>	  
	</xsl:if>	  
      </body>
    </html>
  </xsl:template>

  <!-- MAIN MODE based on the deep null-transform -->

  <xsl:template match="@*|node()">
    <xsl:apply-templates select="@*|node()"/>
  </xsl:template>

  <xsl:template match="@ID">
    <xsl:copy/>
  </xsl:template>
  <xsl:template match="@xml:lang">
    <xsl:copy/>
  </xsl:template>

  <!-- styleCode processing: styleCode can be a list of tokens that
       are being combined into a single css class attribute. To 
       come to a normalized combinarion we sort the tokens. 

       Step 1: combine the attribute supplied codes and additional
       codes in a single token list.

       Step 2: split the token list into XML elements
       
       Step 3: sort the elements and turn into a single combo
       token.
    -->

  <xsl:template match="@styleCode" name="styleCodeAttr">
    <xsl:param name="styleCode" select="."/>
    <xsl:param name="additionalStyleCode" select="/.."/>
    <xsl:param name="allCodes"
      select="normalize-space(concat($additionalStyleCode,' ',$styleCode))"/>
    <xsl:param name="additionalStyleCodeSequence" select="/.."/>
    <xsl:variable name="splitRtf">
      <xsl:if test="$allCodes">
        <xsl:call-template name="splitTokens">
          <xsl:with-param name="text" select="$allCodes"/>
        </xsl:call-template>
      </xsl:if>
      <xsl:for-each select="$additionalStyleCodeSequence">
        <token value="{concat(translate(substring(current(),1,1),
	                                'abcdefghijklmnopqrstuvwxyz',
					'ABCDEFGHIJKLMNOPQRSTUVWXYZ'),
			      substring(current(),2))}"/>
      </xsl:for-each>
    </xsl:variable>
    <xsl:variable name="class">
      <xsl:choose>
        <xsl:when test="function-available('exsl:node-set')">
    	  <xsl:variable name="sortedTokensRtf">
            <xsl:for-each select="exsl:node-set($splitRtf)/token">
              <xsl:sort select="@value"/>
      	      <xsl:copy-of select="."/>
            </xsl:for-each>
    	  </xsl:variable>
    	  <xsl:call-template name="uniqueStyleCodesExsl">
    	    <xsl:with-param name="inRtf" select="$sortedTokensRtf"/>
    	  </xsl:call-template>
        </xsl:when>
        <xsl:when test="function-available('msxsl:node-set')">
    	  <xsl:variable name="sortedTokensRtf">
          <xsl:for-each select="msxsl:node-set($splitRtf)/token">
            <xsl:sort select="@value"/>
    	    <xsl:copy-of select="."/>
          </xsl:for-each>
    	  </xsl:variable>
    	  <xsl:call-template name="uniqueStyleCodesMsxsl">
    	    <xsl:with-param name="inRtf" select="$sortedTokensRtf"/>
    	  </xsl:call-template>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="string-length($class) > 0">
      <xsl:attribute name="class">
        <xsl:value-of select="$class"/>
      </xsl:attribute>
    </xsl:if>
  </xsl:template>

  <xsl:template name="uniqueStyleCodesExsl">
     <xsl:param name="inRtf" select="/.."/>
     <xsl:param name="in" select="exsl:node-set($inRtf)/token[@value]"/>
     <xsl:param name="done" select="/.."/>
     <xsl:if test="$in">
        <xsl:if test="not(contains($done,$in[1]/@value))">
          <xsl:value-of select="$in[1]/@value"/>
        </xsl:if>
        <xsl:call-template name="uniqueStyleCodesExsl">
          <xsl:with-param name="inRtf">
	    <xsl:copy-of select="$in[position()>1]"/>
	  </xsl:with-param>
          <xsl:with-param name="done" select="concat($done,@value)"/>
       </xsl:call-template>
     </xsl:if>
  </xsl:template>
  <xsl:template name="uniqueStyleCodesMsxsl">
     <xsl:param name="inRtf" select="/.."/>
     <xsl:param name="in" select="msxsl:node-set($inRtf)/token[@value]"/>
     <xsl:param name="done" select="/.."/>
     <xsl:if test="$in">
        <xsl:if test="not(contains($done,$in[1]/@value))">
          <xsl:value-of select="$in[1]/@value"/>
        </xsl:if>
       <xsl:call-template name="uniqueStyleCodesMsxsl">
         <xsl:with-param name="inRtf">
	   <xsl:copy-of select="$in[position()>1]"/>
	 </xsl:with-param>
         <xsl:with-param name="done" select="concat($done,@value)"/>
       </xsl:call-template>
     </xsl:if>
  </xsl:template>

  <xsl:template name="splitTokens">
    <xsl:param name="text" select="."/>
    <xsl:param name="firstCode" select="substring-before($text,' ')"/>
    <xsl:param name="restOfCodes" select="substring-after($text,' ')"/>
    <xsl:choose>
      <xsl:when test="$firstCode">
        <token value="{concat(translate(substring($firstCode,1,1),
	                                'abcdefghijklmnopqrstuvwxyz',
					'ABCDEFGHIJKLMNOPQRSTUVWXYZ'),
			      substring($firstCode,2))}"/>
        <xsl:if test="string-length($restOfCodes) > 0">
          <xsl:call-template name="splitTokens">
            <xsl:with-param name="text" select="$restOfCodes"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
        <token value="{concat(translate(substring($text,1,1),
	                                'abcdefghijklmnopqrstuvwxyz',
					'ABCDEFGHIJKLMNOPQRSTUVWXYZ'),
			      substring($text,2))}"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- DOCUMENT MODEL -->

  <xsl:template match="v3:document">
    <xsl:apply-templates select="@*|node()[not(self::v3:text or self::v3:component)]"/>

    <p class="PartHeading">HIGHLIGHTS OF PRESCRIBING INFORMATION</p>
    <div class="Highlights">
      <xsl:apply-templates select="//v3:section[
          v3:code[
		descendant-or-self::*[
		(self::v3:code or self::v3:translation) and
		@codeSystem='2.16.840.1.113883.6.1' and @code='34066-1'
		]
	  ]
	]">
	<xsl:with-param name="mode" select="'highlights'"/>
      </xsl:apply-templates>
      <!-- initialapproval, recent major changes -->
      <xsl:apply-templates mode="highlights" select="node()"/>
    </div>
    
    <p class="PartHeading">COMPREHENSIVE PRESCRIBING INFORMATION INDEX</p>
    <div class="Index">
      <xsl:apply-templates mode="index" select="node()"/>
    </div>

    <p class="PartHeading">COMPREHENSIVE PRESCRIBING INFORMATION</p>
    <div class="Body">
      <xsl:apply-templates select="v3:text|v3:component"/>
    </div>
  </xsl:template>

  <xsl:template match="v3:document/v3:title">
    <p class="DocumentTitle">
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="node()"/>
    </p>
  </xsl:template>

  <xsl:template match="v3:document/v3:title/text()" name="breakDocumentTitleText">
    <xsl:param name="text" select="."/>
    <xsl:param name="firstLine" select="substring-before($text,'&#xa;')"/>
    <xsl:param name="restOfLines" select="substring-after($text,'&#xa;')"/>
    <xsl:choose>
      <xsl:when test="string-length($restOfLines) > 0">
        <xsl:value-of select="$firstLine"/>
	<br/>
	<xsl:call-template name="breakDocumentTitleText">
	  <xsl:with-param name="text" select="$restOfLines"/>
	</xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- MODE HIGHLIGHTS -->
  
  <!-- deep null-transform -->
  <xsl:template mode="highlights" match="/|@*|node()">
    <xsl:apply-templates mode="highlights" select="@*|node()"/>
  </xsl:template>

  <!-- Put out section heading only for main sections that contain highlights elements. -->
  <xsl:template  mode="highlights" match="v3:section[descendant::v3:highlight and not(ancestor::v3:section)]">
    <xsl:param name="standardSection"
      select="$standardSections//v3:section[@code=current()/v3:code/descendant-or-self::*[(self::v3:code or self::v3:translation) and @codeSystem='2.16.840.1.113883.6.1']/@code]"/>
    <xsl:variable name="titles" select="v3:title|$standardSection[1]/v3:title"/>
    <div class="DotleaderTop"/>
    <h1 class="Highlights">
      <xsl:apply-templates mode="mixed" select="$titles[1]/node()"/>
    </h1>
    <div class="DotleaderBot"/>
    <xsl:apply-templates mode="highlights" select="@*|node()[not(self::v3:title)]"/>
  </xsl:template>

  <xsl:template mode="highlights" match="v3:highlight//v3:paragraph">
    <p class="Highlights{@styleCode}">
      <xsl:apply-templates select="v3:caption"/>
      <xsl:apply-templates mode="mixed" 
             select="node()[not(self::v3:caption)]"/>
      <xsl:text> </xsl:text>
      <xsl:if test="not(following-sibling::v3:paragraph)">
        <xsl:variable name="reference" select="ancestor::v3:highlight[1]/v3:reference"/>
        <xsl:apply-templates mode="reference" select=".|//v3:section[v3:id/@root=$reference/v3:section/v3:id/@root and not(ancestor::v3:reference)]"/>
      </xsl:if>
    </p>
  </xsl:template>

  <xsl:template mode="mixed" match="v3:linkHtml[@href]">
    <xsl:param name="target" select="//*[@ID=substring-after(current()/@href,'#')]"/>
    <xsl:if test="contains(@styleCode,'MainTitle') and $target/ancestor-or-self::v3:section[last()]/v3:title">
      <xsl:value-of select="$target/ancestor-or-self::v3:section[last()]/v3:title"/>
      <xsl:text> </xsl:text>
    </xsl:if>
    <xsl:if test="contains(@styleCode,'SubTitle') and $target/v3:title">
      <xsl:if test="contains(@styleCode,'MainTitle') and $target/ancestor-or-self::v3:section[last()]/v3:title">
        <xsl:text>: </xsl:text>
      </xsl:if>
      <xsl:value-of select="$target/v3:title"/>
      <xsl:text> </xsl:text>
    </xsl:if>
    <xsl:apply-templates mode="reference" select="$target"/>
  </xsl:template>

  <xsl:template mode="highlights" match="v3:highlight//v3:paragraph[@styleCode='Bullet']" priority="2">
    <p class="HighlightsHanging">
      <span class="Exdent">&#x2022;</span>
      <xsl:apply-templates select="v3:caption"/>
      <xsl:apply-templates mode="mixed" 
             select="node()[not(self::v3:caption)]"/>
      <xsl:text> </xsl:text>
      <xsl:if test="not(following-sibling::v3:paragraph)">
        <xsl:variable name="reference" select="ancestor::v3:highlight/reference"/>
        <xsl:apply-templates mode="reference" select=".|//v3:section[v3:id/@root=$reference/v3:section/v3:id/@root and v3:title]"/>
      </xsl:if>
    </p>
  </xsl:template>

  <xsl:template mode="highlights" match="v3:tr"/>

  <xsl:template mode="highlights" match="v3:highlight//v3:table">
    <xsl:apply-templates select="."/>
  </xsl:template>

  <!-- MODE index -->
  
  <!-- deep null-transform -->
  <xsl:template mode="index" match="/|@*|node()">
    <xsl:apply-templates mode="index" select="@*|node()"/>
  </xsl:template>

  <xsl:template  mode="index" match="v3:section[v3:title]">
    <xsl:param name="standardSection"
      select="$standardSections//v3:section[@code=current()/v3:code/descendant-or-self::*[(self::v3:code or self::v3:translation) and @codeSystem='2.16.840.1.113883.6.1']/@code]"/>
    <xsl:variable name="titles" select="v3:title|$standardSection[1]/v3:title"/>
    <xsl:variable name="sectionNumberSequence">
      <xsl:apply-templates mode="sectionNumber" select="ancestor-or-self::v3:section"/>
    </xsl:variable>     
    <p class="IndexEntry">
      <span class="SectionNumber">
        <a href="#section-{substring($sectionNumberSequence,2)}">
          <xsl:value-of select="substring($sectionNumberSequence,2)"/>
        </a>
      </span>
      <xsl:apply-templates mode="mixed" select="$titles[1]/node()"/>
      <xsl:apply-templates mode="index" select="@*|node()[not(self::v3:title)]"/>
    </p>
  </xsl:template>

  <!-- MODE: reference -->
  <!-- Create a section number reference such as (13.2) -->
  <xsl:template mode="reference" match="/|@*|node()">
   <xsl:text>(</xsl:text> 
   <xsl:variable name="sectionNumberSequence">
     <xsl:apply-templates mode="sectionNumber" select="ancestor-or-self::v3:section"/>
   </xsl:variable>     
   <a href="#section-{substring($sectionNumberSequence,2)}">
     <xsl:value-of select="substring($sectionNumberSequence,2)"/>
   </a>
   <xsl:text>)</xsl:text>
  </xsl:template>
  
  <!-- FOOTNOTES: PLACE FOOTNOTE MARK HERE -->

  <xsl:template match="v3:footnote">
    <xsl:variable name="number" select="count(preceding::v3:footnote)+1"/>
    <xsl:variable name="mark" select="substring($footnoteMarks,$number,1)"/>
    <a name="footnote-reference-{$number}" href="#footnote-{$number}" class="Sup"><xsl:value-of select="$mark"/></a>
  </xsl:template>
  
  <xsl:template match="v3:flushfootnotes">
    <dl class="Footnote">
      <xsl:apply-templates mode="footnote" select="ancestor-or-self::v3:table[1]"/>
    </dl>
  </xsl:template>

  <xsl:template mode="footnote" match="/|node()">
    <xsl:apply-templates mode="footnote" select="node()"/>
  </xsl:template>
  <xsl:template mode="footnote" match="v3:footnote">
    <xsl:variable name="number" select="count(preceding::v3:footnote)+1"/>
    <xsl:variable name="mark" select="substring($footnoteMarks,$number,1)"/>
    <dt><a name="footnote-{$number}" href="#footnote-reference-{$number}"><xsl:value-of select="$mark"/></a></dt>
    <dd><xsl:apply-templates mode="mixed" select="node()"/></dd>
  </xsl:template>
  <xsl:template mode="footnote" match="v3:rowsFromSubHighlights">
    <xsl:apply-templates mode="footnote" select="ancestor::v3:section[last()]//v3:highlight/v3:text/v3:tr"/>
  </xsl:template>

  <!-- MODE: sectionNumber -->
  <!-- Special mode to construct a section number. Apply to a sequence of sections on the ancestor-or-self axis. -->
  <!-- Shallow null-transform for anything but sections. -->
  <xsl:template mode="sectionNumber" match="/|@*|node()"/>
  <xsl:template mode="sectionNumber" match="v3:section">
    <xsl:param name="standardSection"
      select="$standardSections//v3:section[@code=current()/v3:code/descendant-or-self::*[(self::v3:code or self::v3:translation) and @codeSystem='2.16.840.1.113883.6.1']/@code]"/>
    <xsl:variable name="standardSectionNumber" select="$standardSection/@number"/>
    <xsl:choose>
      <xsl:when test="$standardSectionNumber">
        <xsl:value-of select="concat('.',$standardSectionNumber)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="concat('.',count(parent::v3:component/preceding-sibling::v3:component[child::v3:section])+1)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- SECTION MODEL -->

  <xsl:template match="v3:section">
    <xsl:param name="standardSection"
      select="$standardSections//v3:section[@code=current()/v3:code/descendant-or-self::*[(self::v3:code or self::v3:translation) and @codeSystem='2.16.840.1.113883.6.1']/@code]"/>
    <xsl:param name="sectionLevel" select="count(ancestor-or-self::v3:section)"/>
    <xsl:variable name="sectionNumberSequence">
      <xsl:apply-templates mode="sectionNumber" select="ancestor-or-self::v3:section"/>
    </xsl:variable>
    <xsl:variable name="titles" select="v3:title"/> <!-- don't use |$standardSection[1]/v3:title"/-->
    <a name="section-{substring($sectionNumberSequence,2)}"/>
    <p/><!-- this funny p is used to prevent melting two sub-sections together in condensed style -->
    <xsl:apply-templates select="$titles[1]">
      <xsl:with-param name="sectionLevel" select="$sectionLevel"/>
      <xsl:with-param name="sectionNumber" select="substring($sectionNumberSequence,2)"/>
    </xsl:apply-templates>
    <xsl:if test="$show-data">
      <xsl:apply-templates mode="data" select="."/>
    </xsl:if>
    <xsl:apply-templates select="@*|node()[not(self::v3:title)]"/>
  </xsl:template>

  <!-- boxed warning -->

  <xsl:template match="v3:section[
          v3:code[
		descendant-or-self::*[
		(self::v3:code or self::v3:translation) and
		@codeSystem='2.16.840.1.113883.6.1' and @code='34066-1'
		]
	  ]
	]" priority="2">
    <xsl:param name="mode" select="/.."/>
    <xsl:param name="standardSection"
      select="$standardSections//v3:section[@code=current()/v3:code/descendant-or-self::*[(self::v3:code or self::v3:translation) and @codeSystem='2.16.840.1.113883.6.1']/@code]"/>
    <xsl:param name="sectionLevel" select="count(ancestor-or-self::v3:section)"/>
    <div>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode" select="'Warning'"/>
      </xsl:call-template>
      <xsl:variable name="sectionNumberSequence">
        <xsl:apply-templates mode="sectionNumber" select="ancestor-or-self::v3:section"/>
      </xsl:variable>
      <xsl:variable name="titles" select="v3:title"/> <!-- don't use |$standardSection[1]/v3:title"/ -->
      <xsl:if test="not($mode='highlights')">
        <a name="section-{substring($sectionNumberSequence,2)}"/>
      </xsl:if>
      <p/><!-- this funny p is used to prevent melting two sub-sections together in condensed style -->
      <xsl:apply-templates select="$titles[1]">
        <xsl:with-param name="sectionLevel" select="$sectionLevel"/>
        <xsl:with-param name="sectionNumber" select="substring($sectionNumberSequence,2)"/>
      </xsl:apply-templates>
      <xsl:apply-templates select="@*|node()[not(self::v3:title)]"/>
    </div>
  </xsl:template>

  <xsl:template match="v3:section/v3:title">
    <xsl:param name="sectionLevel" select="count(ancestor::v3:section)"/>
    <xsl:param name="sectionNumber" select="/.."/>
    <xsl:element name="h{$sectionLevel}">
      <xsl:apply-templates select="@*"/>
      <xsl:if test="$sectionNumber">
        <span class="SectionNumber">
          <xsl:value-of select="$sectionNumber"/>
        </span>
      </xsl:if>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="v3:section/v3:text">
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates mode="mixed" select="node()"/>
  </xsl:template>

  <!-- DISPLAY SUBJECT STRUCTURED INFORMATION -->

  <xsl:template match="v3:section/v3:subject">
    <xsl:if test="$show-data">
      <div class="Subject">
        <xsl:apply-templates mode="data" select="node()"/>
      </div>
    </xsl:if>
    <xsl:if test="$show-subjects-xml">
      <div class="Subject"
           onclick="xmlVerbatimClick(event);"
           ondblclick="xmlVerbatimDblClick(event);">
        <xsl:apply-templates mode="xml-verbatim" select="node()"/>
      </div>
    </xsl:if>
  </xsl:template>

  <xsl:template match="v3:highlight">
    <xsl:if test="$show-data or $show-subjects-xml">
    <table class="HighlightTable">
      <tr class="HighlightTableTextrow">
        <td><xsl:apply-templates select="@*|node()"/></td>
      </tr>
      <xsl:if test="$show-data">
        <tr class="HighlightTableDatarow">
	  <td><xsl:apply-templates mode="data" select="node()"/></td>
	</tr>
      </xsl:if>
      <xsl:if test="$show-subjects-xml">
        <tr class="HighlightTableXMLRow">
	  <td>
            <div onclick="xmlVerbatimClick(event);" ondblclick="xmlVerbatimDblClick(event);">
              <xsl:apply-templates mode="xml-verbatim" select="."/>
            </div>
	  </td>
	</tr>
      </xsl:if>
    </table>
    </xsl:if>
  </xsl:template>

  <xsl:template mode="xml-verbatim-ellips" match="v3:component|v3:subject|v3:subjectOf|v3:componentOf">
    <xsl:variable name="content" select="*[last()]"/>
    <xsl:text> ... </xsl:text>
    <xsl:value-of select="local-name($content)"/>
    <xsl:apply-templates mode="xml-verbatim-ellips" select="$content"/>
  </xsl:template>

  <xsl:template mode="xml-verbatim-ellips" match="v3:section">
    <xsl:apply-templates mode="xml-verbatim-ellips" select="v3:title"/>
  </xsl:template>

  <xsl:template mode="xml-verbatim-ellips" match="v3:title|v3:caption">
    <xsl:variable name="text" select="text()"/>
    <xsl:if test="$text">
      <xsl:text> ... </xsl:text>
      <xsl:value-of select="$text"/>
    </xsl:if>
  </xsl:template>

  <xsl:template mode="xml-verbatim-ellips" match="node()[node()[1][self::text()]]">
    <xsl:variable name="text" select="substring(text(),1,25)"/>
    <xsl:if test="$text">
      <xsl:text> ... </xsl:text>
      <xsl:value-of select="$text"/>
    </xsl:if>
  </xsl:template>

  <xsl:template mode="xml-verbatim-ellips" match="node()[node()[1][self::v3:caption]]">
    <xsl:apply-templates mode="xml-verbatim-ellips" select="node()"/>
  </xsl:template>

  <!-- PARAGRAPH MODEL -->

  <xsl:template match="v3:paragraph">
    <p>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode">
	  <xsl:if test="count(preceding-sibling::v3:paragraph)=0">
	    <xsl:text>First</xsl:text>
	  </xsl:if>
	</xsl:with-param>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]|v3:caption"/>
      <xsl:apply-templates mode="mixed" select="node()[not(self::v3:caption)]"/>
    </p>
  </xsl:template>

  <xsl:template match="v3:paragraph/v3:caption">
    <span>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode" select="'ParagraphCaption'"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </span>
  </xsl:template>

  <xsl:template match="v3:paragraph[contains(@styleCode,'Footnote') and v3:caption]">
    <dl class="Footnote">
      <dt><xsl:apply-templates mode="mixed" select="node()[self::v3:caption]"/></dt>
      <dd><xsl:apply-templates mode="mixed" select="node()[not(self::v3:caption)]"/></dd>
    </dl>
  </xsl:template> 

  <!-- LIST MODEL -->

  <!-- listType='unordered' is default, if any item has a caption,
       all should have a caption -->
  <xsl:template match="v3:list[not(v3:item/v3:caption)]">
    <xsl:apply-templates select="v3:caption"/> 
    <ul>
      <xsl:apply-templates select="@*|node()[not(self::v3:caption)]"/>
    </ul>
  </xsl:template>
  <xsl:template match="v3:list[@listType='ordered' and
       not(v3:item/v3:caption)]" priority="1">
    <xsl:apply-templates select="v3:caption"/> 
    <ol>
      <xsl:apply-templates select="@*|node()[not(self::v3:caption)]"/>
    </ol>
  </xsl:template>
  <xsl:template match="v3:list/v3:item[not(parent::v3:list/v3:item/v3:caption)]">
    <li>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </li>
  </xsl:template>

  <!-- lists with custom captions -->
  <xsl:template match="v3:list[v3:item/v3:caption]">
    <xsl:apply-templates select="v3:caption"/> 
    <dl>
      <xsl:apply-templates select="@*|node()[not(self::v3:caption)]"/>
    </dl>
  </xsl:template>
  <xsl:template match="v3:list/v3:item[parent::v3:list/v3:item/v3:caption]">
    <xsl:apply-templates select="v3:caption"/>
    <dd>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates mode="mixed" select="node()[not(self::v3:caption)]"/>
    </dd>
  </xsl:template>
  <xsl:template match="v3:list/v3:item/v3:caption">
    <dt>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </dt>
  </xsl:template>

  <xsl:template match="v3:list/v3:caption">
    <p>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode" select="'ListCaption'"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </p>
  </xsl:template> 

  <!-- TABLE MODEL -->

  <xsl:template match="v3:table">
    <table>
      <xsl:apply-templates select="@*|node()"/>
    </table>
  </xsl:template>
  <xsl:template match="v3:table/@summary
                      |v3:table/@width
                      |v3:table/@border
                      |v3:table/@frame
                      |v3:table/@rules
                      |v3:table/@cellspacing
                      |v3:table/@cellpadding">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="v3:table/v3:caption">
    <caption>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </caption>
  </xsl:template> 
  
  <xsl:template match="v3:thead">
    <thead>
      <xsl:apply-templates select="@*|node()"/>
    </thead>
  </xsl:template>
  <xsl:template match="v3:thead/@align
                      |v3:thead/@char
                      |v3:thead/@charoff
                      |v3:thead/@valign">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="v3:tfoot">
    <tfoot>
      <xsl:apply-templates select="@*|node()"/>
    </tfoot>
  </xsl:template>

  <xsl:template match="v3:tfoot/@align
                      |v3:tfoot/@char
                      |v3:tfoot/@charoff
                      |v3:tfoot/@valign">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="v3:tbody">
    <tbody>
      <xsl:apply-templates select="@*|node()"/>
    </tbody>
  </xsl:template>
  <xsl:template match="v3:tbody[not(preceding-sibling::v3:thead)]">
    <tbody>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode" select="'Headless'"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates select="node()"/>
    </tbody>
  </xsl:template>
  <xsl:template match="v3:tbody/@align
                      |v3:tbody/@char
                      |v3:tbody/@charoff
                      |v3:tbody/@valign">
    <xsl:copy-of select="."/>
  </xsl:template>

  <!-- specially assembled tables where the bodies are put together from sub-elements -->
  <xsl:template match="v3:tbody[@styleCode='continued' and @ID]" priority="1">
    <tbody>
      <xsl:apply-templates select="@*|node()"/>
      <xsl:apply-templates select="ancestor::v3:section[last()]//v3:tbody[@styleCode='continuation' and starts-with(@ID,current()/@ID)]/node()"/>
    </tbody>
  </xsl:template>
  <xsl:template match="v3:tbody[not(preceding-sibling::v3:thead) and @styleCode='continued' and @ID]" priority="1">
    <tbody>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode" select="'Headless'"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates select="node()"/>
      <xsl:apply-templates select="ancestor::v3:section[last()]//v3:tbody[@styleCode='continuation' and starts-with(@ID,current()/@ID)]/node()"/>
    </tbody>
  </xsl:template>
  <xsl:template match="v3:tbody[@styleCode='continuation']" priority="1"/>
  <xsl:template match="v3:table[@styleCode='continuation']" priority="1"/>
  <xsl:template match="v3:tfoot[@styleCode='continuation']" priority="1"/>
  <xsl:template match="v3:thead[@styleCode='continuation']" priority="1"/>
  <xsl:template match="v3:*[@styleCode='phantom']" priority="1"/>
  <!-- end of special continued tables handling -->

  <xsl:template match="v3:tr">
    <tr>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode">
          <xsl:if test="position()=1"><xsl:text> First </xsl:text></xsl:if>
          <xsl:if test="position()=last()"><xsl:text> Last </xsl:text></xsl:if>
        </xsl:with-param>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates select="node()"/>
    </tr>
  </xsl:template>
  <xsl:template match="v3:tr/@align
                      |v3:tr/@char
                      |v3:tr/@charoff
                      |v3:tr/@valign">
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <xsl:template match="v3:th">
    <!-- determine our position to find out the associated col -->
    <xsl:param name="position"
       select="1+count(preceding-sibling::v3:td[not(@colspan)]
                    |preceding-sibling::v3:th[not(@colspan)])
               +sum(preceding-sibling::v3:td/@colspan
	            |preceding-sibling::v3:th/@colspan)"/> 
    <xsl:param name="associatedCol" 
       select="(ancestor::v3:table/v3:colgroup/v3:col|ancestor::v3:table/v3:col)[$position]"/>
    <xsl:param name="associatedColgroup" select="$associatedCol/parent::v3:colgroup"/>
    <th>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode">
          <xsl:if test="not(ancestor::v3:tfoot) 
	            and contains($associatedColgroup/@styleCode,'Lrule') 
                    and not($associatedCol/preceding-sibling::v3:col)">
            <xsl:text> Lrule </xsl:text>
          </xsl:if>    
          <xsl:if test="not(ancestor::v3:tfoot) 
                    and contains($associatedColgroup/@styleCode,'Rrule') 
		    and not($associatedCol/following-sibling::v3:col)">
            <xsl:text> Rrule </xsl:text>
          </xsl:if>    
	</xsl:with-param>
      </xsl:call-template>
      <xsl:copy-of select="$associatedCol/@align"/>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </th>
  </xsl:template>
  <xsl:template match="v3:th/@align
                      |v3:th/@char
                      |v3:th/@charoff
                      |v3:th/@valign
                      |v3:th/@abbr
                      |v3:th/@axis
                      |v3:th/@headers
                      |v3:th/@scope
                      |v3:th/@rowspan
                      |v3:th/@colspan">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="v3:td">
    <!-- determine our position to find out the associated col -->
    <xsl:param name="position"
       select="1+count(preceding-sibling::v3:td[not(@colspan)]
                    |preceding-sibling::v3:th[not(@colspan)])
               +sum(preceding-sibling::v3:td/@colspan
	            |preceding-sibling::v3:th/@colspan)"/> 
    <xsl:param name="associatedCol" 
       select="(ancestor::v3:table/v3:colgroup/v3:col|ancestor::v3:table/v3:col)[$position]"/>
    <xsl:param name="associatedColgroup" select="$associatedCol/parent::v3:colgroup"/>
    <td>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode">
          <xsl:if test="not(ancestor::v3:tfoot) 
	            and contains($associatedColgroup/@styleCode,'Lrule') 
                    and not($associatedCol/preceding-sibling::v3:col)">
            <xsl:text> Lrule </xsl:text>
          </xsl:if>    
          <xsl:if test="not(ancestor::v3:tfoot) 
                    and contains($associatedColgroup/@styleCode,'Rrule') 
		    and not($associatedCol/following-sibling::v3:col)">
            <xsl:text> Rrule </xsl:text>
          </xsl:if>    
	</xsl:with-param>
      </xsl:call-template>
      <xsl:copy-of select="$associatedCol/@align"/>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </td>
  </xsl:template>
  <xsl:template match="v3:td/@align
                      |v3:td/@char
                      |v3:td/@charoff
                      |v3:td/@valign
                      |v3:td/@abbr
                      |v3:td/@axis
                      |v3:td/@headers
                      |v3:td/@scope
                      |v3:td/@rowspan
                      |v3:td/@colspan">
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <xsl:template match="v3:colgroup">
    <colgroup><xsl:apply-templates select="@*|node()"/></colgroup>
  </xsl:template>
  <xsl:template match="v3:colgroup/@span
                      |v3:colgroup/@width
                      |v3:colgroup/@align
                      |v3:colgroup/@char
                      |v3:colgroup/@charoff
                      |v3:colgroup/@valign">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="v3:col">
    <col><xsl:apply-templates select="@*|node()"/></col>
  </xsl:template>
  <xsl:template match="v3:col/@span
                      |v3:col/@width
                      |v3:col/@align
                      |v3:col/@char
                      |v3:col/@charoff
                      |v3:col/@valign">
    <xsl:copy-of select="."/>
  </xsl:template>


  <!-- MIXED MODE: where text is rendered as is, even if nested 
         inside elements that we do not understand  --> 

  <!-- based on the deep null-transform -->
  <xsl:template mode="mixed" match="@*|node()">
    <xsl:apply-templates mode="mixed" select="@*|node()"/>
  </xsl:template>

  <xsl:template mode="mixed" match="text()" priority="0">
    <xsl:copy/>
  </xsl:template>

  <xsl:template mode="mixed" match="v3:content">
    <span>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCodeSequence" select="@emphasis|@revised"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </span>
  </xsl:template>
  <xsl:template mode="mixed" match="v3:content[@emphasis='yes']" priority="1">
    <em>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCodeSequence" select="@revised"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </em>
  </xsl:template>
  <xsl:template mode="mixed" match="v3:content[@emphasis]">
    <em>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCodeSequence" 
	  select="@emphasis|@revised"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </em>
  </xsl:template>

  <!-- We don't use <sub> and <sup> elements here because IE produces
       ugly uneven line spacing. -->
  <xsl:template mode="mixed" match="v3:sub">
    <span class="Sub">
      <xsl:apply-templates mode="mixed" select="@*|node()"/>
    </span>
  </xsl:template>
  <xsl:template mode="mixed" match="v3:sup">
    <span class="Sup">
      <xsl:apply-templates mode="mixed" select="@*|node()"/>
    </span>
  </xsl:template>
  <xsl:template mode="mixed" match="v3:br">
    <br/>
  </xsl:template>

  <xsl:template mode="mixed" priority="1"
       match="v3:renderMultiMedia[@referencedObject
		  and (   ancestor::v3:paragraph
		       or ancestor::v3:td
		       or ancestor::v3:th)]">
    <xsl:variable name="reference" select="@referencedObject"/>
    <img src="{//v3:observationMedia[@ID=$reference]//v3:reference/@value}">
      <xsl:apply-templates select="@*"/>
    </img>
  </xsl:template>

  <xsl:template mode="mixed" 
      match="v3:renderMultiMedia[@referencedObject]">
    <xsl:variable name="reference" select="@referencedObject"/>
    <div>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode" select="'Figure'"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <img src="{//v3:observationMedia[@ID=$reference]//v3:reference/@value}"/>
      <xsl:apply-templates select="v3:caption"/>
    </div>
  </xsl:template>
  <xsl:template match="v3:renderMultiMedia/v3:caption">
    <p>
      <xsl:call-template name="styleCodeAttr">
        <xsl:with-param name="styleCode" select="@styleCode"/>
        <xsl:with-param name="additionalStyleCode" select="'MultiMediaCaption'"/>
      </xsl:call-template>
      <xsl:apply-templates select="@*[not(local-name(.)='styleCode')]"/>
      <xsl:apply-templates mode="mixed" select="node()"/>
    </p>
  </xsl:template> 
  
  <!-- EXIT MIXED MODE AT DEFINED POINTS -->
  <xsl:template mode="mixed" match="v3:paragraph|v3:list|v3:table|v3:footnote|v3:flushfootnotes">
    <xsl:apply-templates select="."/>
  </xsl:template>

  <!-- MODE: data -->

  <xsl:template mode="data" match="/|@*|node()">
    <xsl:apply-templates mode="data" select="@*|node()"/>
  </xsl:template>

  <xsl:template mode="data" match="v3:substanceAdministration/v3:reason/v3:indicationObservationCriterion">
    <p class="Data"><span class="ParagraphCaption">Indication:</span> <xsl:apply-templates mode="data" select="@*|node()"/></p>
  </xsl:template>

  <xsl:template mode="data" match="v3:substanceAdministration/v3:subjectOf/v3:issue">
    <p class="Data"><span class="ParagraphCaption">Issue:</span> <xsl:apply-templates mode="data" select="@*|node()"/></p>
  </xsl:template>

  <xsl:template mode="data" match="v3:highlight//v3:text"/>

  <xsl:template mode="data" match="v3:issue/v3:subject/v3:clinicalSituationCriterion">
    <xsl:text> in patients with </xsl:text>
    <xsl:apply-templates mode="data" select="@*|node()"/>
  </xsl:template>

  <xsl:template mode="data" match="v3:issue/v3:subject/v3:substanceAdministrationCriterion">
    <xsl:text> in patients treated with </xsl:text>
    <xsl:apply-templates mode="data" select="@*|node()"/>
  </xsl:template>

  <xsl:template mode="data" match="v3:issue/v3:risk/v3:consequenceObservation">
    <xsl:text> may lead to </xsl:text>
    <xsl:apply-templates mode="data" select="@*|node()"/>
  </xsl:template>

  <xsl:template mode="data" match="v3:issue/v3:precondition">
    <xsl:text> when </xsl:text>
    <xsl:apply-templates mode="data" select="@*|node()"/>
    <xsl:text> is present </xsl:text>
  </xsl:template>

  <!-- block at sections unless handled specially -->
  <xsl:template mode="data" match="v3:section"/>

  <!-- special handling of DESCRIPTION section to add data element view -->
  <xsl:template mode="data" match="v3:section[
          v3:code[
		descendant-or-self::*[
		(self::v3:code or self::v3:translation) and
		@codeSystem='2.16.840.1.113883.6.1' and @code='34089-3'
		]
	  ]
	]">
      <xsl:variable name="data" 
         select="ancestor::v3:structuredBody//v3:section/v3:subject"/>
      <xsl:variable name="products" 
         select="$data/v3:manufacturedProduct/v3:medicine"/>
      <xsl:for-each select="$products">
        <xsl:variable name="product" select="current()"/>
        <xsl:variable name="productName" select="$product/v3:name"/>
        <xsl:if test="not(preceding::v3:manufacturedProduct[v3:medicine/v3:name=$productName])">
	  <!-- this could have done just by way of the initial products XPath, but in case we want to group by
	       more than one feature it is more extensible with this if block. -->
	  <xsl:variable name="productsWithSameName"
             select="$products[v3:name=$productName]"/>
	  <div class="Subject">
	    <table>
	      <col align="left"/>
	      <col align="left"/>
	      <tr>
	  	 <th>Proprietary name: </th>
	  	 <td><xsl:apply-templates mode="data" select="$product/v3:name"/></td>
	      </tr>
	      <tr>
	  	 <th>Established name: </th>
	  	 <td>
	  	   <xsl:apply-templates mode="data" select="$product/v3:generic/v3:genericMedicine/v3:name"/>
		   <!-- NOTICE this consistency check spans the different routes! -->
	  	   <xsl:for-each select="$products[v3:name=$productName and (
	  				   not(./v3:generic/v3:genericMedicine/v3:name
	  			      = $product/v3:generic/v3:genericMedicine/v3:name))]">
	  	     <br/>
	  	     <strong>
	  	       <xsl:text>MISMATCHED DATA: </xsl:text>
	  	       <xsl:apply-templates mode="data" select="./v3:generic/v3:genericMedicine/v3:name"/>
	  	     </strong>
	  	   </xsl:for-each>
	  	 </td>
	      </tr>
	      <tr>
	  	 <th>Route of administration: </th>
	  	 <td>
		   <xsl:variable name="routeCodes" select="$product/../v3:consumedIn/v3:substanceAdministration/v3:routeCode"/>
		   <!-- could be multiple routes here -->
		   <xsl:for-each select="$routeCodes">
		     <xsl:variable name="routeCode" select="."/>
		     <xsl:if test="position()>1">
		       <xsl:text>, </xsl:text>
		     </xsl:if>
	  	     <xsl:apply-templates mode="data" select="$routeCode"/>
		   </xsl:for-each>
	  	 </td>
	      </tr>
	      <tr>
	  	 <th>Active ingredients (moiety): </th>
	  	 <td>
	  	   <xsl:for-each select="$product/v3:activeIngredient/v3:substance">
	  	     <xsl:variable name="ingredient" select="current()"/>
	  	     <xsl:variable name="activeMoiety" select="$ingredient/v3:activeMoiety/v3:activeMoiety"/>
	  	     <xsl:variable name="quantity" select="$ingredient/../v3:quantity"/>
	  	     <xsl:if test="position() > 1">
	  	       <xsl:text>, </xsl:text>
	  	     </xsl:if>
	  	     <xsl:apply-templates mode="data" select="$ingredient/v3:name"/>
	  	     <xsl:if test="$activeMoiety">
	  	       <xsl:text> (</xsl:text>
	  	       <xsl:apply-templates mode="data" select="$activeMoiety/v3:name"/>
	  	       <xsl:text>)</xsl:text>
	  	     </xsl:if>
	  	     <xsl:for-each select="$products[v3:name=$productName and (
	  				   not(./v3:activeIngredient/v3:substance/v3:name = $ingredient/v3:name)
	  			       or ($ingredient/v3:code and
	  				   not(./v3:activeIngredient/v3:substance/v3:code/@code = $ingredient/v3:code/@code
	  				   and ./v3:activeIngredient/v3:substance/v3:code/@codeSystem = $ingredient/v3:code/@codeSystem)))]">
	  	       <br/>
	  	       <strong>
	  		 <xsl:text>MISMATCHED DATA: </xsl:text>
	  		 <xsl:apply-templates mode="data" select="$ingredient/v3:name"/>
	  		 <xsl:text> NOT IN ALL PRODUCT VARIANTS</xsl:text>
	  	       </strong>
	  	     </xsl:for-each>
	  	   </xsl:for-each>
	  	   <xsl:for-each select="$products[v3:name=$productName]/v3:activeIngredient/v3:substance[
	  				   not(v3:name = $product/v3:activeIngredient/v3:substance/v3:name)
	  			       or (v3:code and
	  				   not(v3:code/@code = $product/v3:activeIngredient/v3:substance/v3:code/@code
	  				   and v3:code/@codeSystem = $product/v3:activeIngredient/v3:substance/v3:code/@codeSystem))]">
	  	     <br/>
	  	     <strong>
	  	       <xsl:text>MISMATCHED DATA: </xsl:text>
	  	       <xsl:apply-templates mode="data" select="./v3:name"/>
	  	       <xsl:text> FOUND IN SOME BUT NOT ALL PRODUCT VARIANTS</xsl:text>
	  	     </strong>
	  	   </xsl:for-each>
	  	 </td>
	      </tr>
	      <!-- tr>
	  	 <th>Inactive ingredients: </th>
	  	 <td>
	  	   <xsl:for-each select="$product/v3:inactiveIngredient/v3:substance">
	  	     <xsl:variable name="ingredient" select="current()"/>
	  	     <xsl:variable name="quantity" select="$ingredient/../v3:quantity"/>
	  	     <xsl:if test="position() > 1">
	  	       <xsl:text>, </xsl:text>
	  	     </xsl:if>
	  	     <xsl:apply-templates mode="data" select="$ingredient/v3:name"/>
	  	   </xsl:for-each>
	  	   <xsl:if test="count($productsWithSameName) > 1">
	  	     <br/>(SOME PRODUCTS OF THE SAME FAMILY MAY HAVE OTHER INACTIVE INGREDIENTS)
	  	   </xsl:if>
	  	 </td>
	      </tr -->
	    </table>
	  
	    <table>
	      <col align="left"/>
	      <col align="left"/>
	      <col align="left"/>
	      <tr>
	  	 <th>#</th>
	  	 <th>Strength</th>
		 <th>Form</th>
	  	 <th>Inactive ingredients</th>
	      </tr>
	      <xsl:for-each select="$productsWithSameName">
	  	 <xsl:variable name="productVariant" select="current()"/>
	  	 <xsl:variable name="productVariantNumber" select="position()"/>
	  	 <tr>
	  	   <td><xsl:value-of select="$productVariantNumber"/></td>
	  	   <td>
	  	     <xsl:for-each select="$productVariant/v3:activeIngredient/v3:substance">
	  	       <xsl:variable name="ingredient" select="current()"/>
	  	       <xsl:variable name="activeMoiety" select="$ingredient/v3:activeMoiety/v3:activeMoiety"/>
	  	       <xsl:variable name="quantity" select="$ingredient/../v3:quantity"/>
	  	       <xsl:if test="position() > 1">
	  		 <xsl:text>, </xsl:text>
	  		 <br/>
	  	       </xsl:if>
	  	       <!-- xsl:apply-templates mode="data" select="$ingredient/v3:name"/>
	  	       <xsl:if test="$activeMoiety">
	  		 <xsl:text> (</xsl:text>
	  		 <xsl:apply-templates mode="data" select="$activeMoiety/v3:name"/>
	  		 <xsl:text>)</xsl:text>
	  	       </xsl:if>
	  	       <xsl:text> </xsl:text -->
	  	       <xsl:apply-templates mode="data" select="$quantity"/>
	  	     </xsl:for-each>
		   </td>
		   <td>
	  	     <xsl:variable name="form" select="$productVariant/v3:formCode"/>
	  	     <xsl:apply-templates mode="data" select="$form"/>
	  	   </td>
	  	   <td>
	  	     <xsl:for-each select="$productVariant/v3:inactiveIngredient/v3:substance">
	  	       <xsl:variable name="ingredient" select="current()"/>
	  	       <xsl:variable name="quantity" select="$ingredient/../v3:quantity"/>
	  	       <xsl:if test="position() > 1">
	  		 <xsl:text>, </xsl:text>
	  	       </xsl:if>
	  	       <xsl:apply-templates mode="data" select="$ingredient/v3:name"/>
	  	     </xsl:for-each>
	  	   </td>
	  	 </tr>
	      </xsl:for-each>
	    </table>
	  </div>
	</xsl:if>
      </xsl:for-each>
  </xsl:template>

  <!-- special handling of HOW SUPPLIED section to add data element view -->
  <xsl:template mode="data" match="v3:section[
          v3:code[
		descendant-or-self::*[
		(self::v3:code or self::v3:translation) and
		@codeSystem='2.16.840.1.113883.6.1' and @code='34069-5'
		]
	  ]
	]">
    <xsl:if test="$show-data">
      <xsl:variable name="data" 
         select="ancestor::v3:structuredBody//v3:section/v3:subject"/>
      <xsl:variable name="products" 
         select="$data/v3:manufacturedProduct/v3:medicine"/>
      <div class="Subject">
        <table>
          <col align="left"/>
          <col align="left"/>
          <col align="left"/>
          <col align="left"/>
          <col align="left"/>
          <col align="left"/>
          <col align="left"/>
          <col align="left"/>
	  <tr>
	    <th>#</th>
            <th>Name</th>
	    <th>Strength</th>
	    <th>Dosage Form</th>
	    <th>Appearance</th>
            <th>Package Type</th>
	    <th>Package Qty</th>
	    <th>NDC</th>
          </tr>
          <xsl:for-each select="$products">
            <xsl:variable name="product" select="current()"/>
	    <xsl:variable name="productNumber" select="position()"/>
	    <xsl:for-each select="$product/v3:container/v3:containingPackagedMedicine">
	      <xsl:variable name="package" select="current()"/>
   	      <tr>
	        <td><xsl:value-of select="$productNumber"/></td>
	        <td>
		  <xsl:apply-templates mode="data" select="$product/v3:name"/>
		</td>
		<td>
                  <xsl:for-each select="$product/v3:activeIngredient/v3:substance">
	            <xsl:variable name="ingredient" select="current()"/>
	            <xsl:variable name="quantity" select="$ingredient/../v3:quantity"/>
	            <xsl:if test="position() > 1">
	              <xsl:text>, </xsl:text>
                    </xsl:if>
	            <xsl:apply-templates mode="data" select="$quantity"/>
	          </xsl:for-each>
                </td>
	        <td>
		  <xsl:apply-templates mode="data" select="$product/v3:formCode"/>
		</td>
		<td>
		  <xsl:for-each select="$product/ancestor::v3:section[1]/v3:component1//v3:observation">
		    <xsl:variable name="observation" select="current()"/>
		    <xsl:if test="position() > 1">
		      <xsl:text>, </xsl:text><br/>
		    </xsl:if>
		    <xsl:apply-templates mode="data" select="$observation/v3:code"/>
		    <xsl:text> = </xsl:text>
		    <xsl:apply-templates mode="data" select="$observation/v3:value"/>
		  </xsl:for-each>
                </td>
	        <td>
  	          <xsl:apply-templates mode="data" select="$package/v3:formCode"/>
		</td>
	        <td>
  	          <xsl:apply-templates mode="data" select="$package/../v3:quantity"/>
		</td>
	        <td>
  	          <xsl:apply-templates mode="data" 
		     select="$package/v3:code[@codeSystem='2.16.840.1.113883.6.69']/@code"/>
		</td>
              </tr>
	    </xsl:for-each>
          </xsl:for-each>
        </table>
      </div>
    </xsl:if>
  </xsl:template>

  <xsl:template mode="data" match="*">
    <xsl:apply-templates mode="data" select="node()"/>
  </xsl:template>
  <xsl:template mode="data" match="text()">
    <xsl:copy/>
  </xsl:template>
  <xsl:template mode="data" match="*[@displayName and not(@code)]">
    <xsl:value-of select="@displayName"/>
  </xsl:template>
  <xsl:template mode="data" match="*[not(@displayName) and @code]">
    <xsl:value-of select="@code"/>
  </xsl:template>
  <xsl:template mode="data" match="*[@displayName and @code]">
    <xsl:value-of select="@displayName"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="@code"/>
    <xsl:text>)</xsl:text>
  </xsl:template>
  <xsl:template mode="data" match="*[@value and not(@unit)]">
    <xsl:value-of select="@value"/>
  </xsl:template>
  <xsl:template mode="data" match="*[@value and @unit]">
    <xsl:value-of select="@value"/>
    <xsl:text>&#160;</xsl:text>
    <xsl:value-of select="@unit"/>
  </xsl:template>
  <xsl:template mode="data" match="*[v3:numerator]">
    <xsl:apply-templates mode="data" select="v3:numerator"/>
    <xsl:if test="v3:denominator[not(@value='1' and (not(@unit) or not(@unit='1')))]">
      <xsl:text> : </xsl:text>
      <xsl:apply-templates mode="data" select="v3:denominator"/>
    </xsl:if>
  </xsl:template>


</xsl:transform>
